import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes
import Qt.labs.qmlmodels

import "../components"
import "./view"

import Artix.Core
import Artix.Gui
import Artix.Models

/**
 * @brief Форма закрытия смены
 */
BasicView {
    contentItem: RowLayout {
        // Левая сторона
        GroupBox {
            Layout.fillHeight: true
            Layout.fillWidth: true
            Style.class: "positionsListBackground"
            ColumnLayout {
                anchors.centerIn: parent
                BasicLabel {
                    Style.class: "shiftCloseTitle"
                    Layout.maximumWidth: 350
                    Layout.alignment: Qt.AlignHCenter
                    horizontalAlignment: Text.AlignHCenter
                    Tr { code: "shiftCloseFormTitle"; defaultMessage: "Закрываем смену" }
                }
                BasicLabel {
                    Style.class: "shiftCloseMessage"
                    Layout.maximumWidth: 350
                    Layout.alignment: Qt.AlignHCenter
                    horizontalAlignment: Text.AlignHCenter
                    Tr { code: "shiftCloseMessage"; defaultMessage: "Не выключайте кассу! Дождитесь завершения операции" }
                }
            }
        }
        // Правая сторона
        Item {
            Layout.preferredWidth: Math.floor((screenWidth * 0.19 + 374) / 8) * 8
            Layout.fillHeight: true
            TopPanel {
                id: topPanel
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
            }
            ListView {
                id: listView
                anchors { left: parent.left; right: parent.right }
                anchors.leftMargin: 56
                spacing: 24
                height: count * 32 + (count - 1) * 24
                anchors.verticalCenter: parent.verticalCenter
                model: ShiftCloseModel { onActionUpdated: listView.positionViewAtEnd() }
                delegate: RowLayout {
                    Item {
                        Layout.preferredWidth: 32
                        Layout.preferredHeight: 32
                        BusyIndicator {
                            padding: 0
                            anchors.fill: parent
                            id: indicator
                            running: model.state === ShiftCloseModel.Running
                            NumberAnimation on opacity { duration: 300 }
                            property real circleWidth: 4
                            background: Rectangle {
                                id: background
                                color: "transparent"
                                border.width: indicator.circleWidth
                                border.color: Style.parameter("color.grayBorder")
                                radius: width
                            }
                            contentItem: Shape {
                                visible: indicator.running
                                ShapePath {
                                    id: shape
                                    strokeColor: Style.parameter("color.grayMedium")
                                    strokeWidth: indicator.circleWidth
                                    fillColor: "transparent"
                                    capStyle: ShapePath.RoundCap
                                    PathAngleArc {
                                        centerX: indicator.width / 2; centerY: indicator.height / 2
                                        radiusX: indicator.width / 2 - shape.strokeWidth / 2; radiusY: radiusX
                                        startAngle: 0
                                        sweepAngle: 45
                                    }
                                }
                                RotationAnimator {
                                    target: indicator.contentItem
                                    running: indicator.running
                                    from: 0; to: 360; loops: Animation.Infinite
                                    duration: 3000
                                }
                            }
                        }
                        Image {
                            id: image
                            anchors.fill: parent
                            source: Style.parameter(icon)
                            fillMode: Image.PreserveAspectFit
                        }
                        state: {
                            if (model.state === ShiftCloseModel.Waiting || model.state === ShiftCloseModel.Running)
                                return "notFinished"
                            return "finished"
                        }
                        states: [
                            State {
                                name: "notFinished"
                                PropertyChanges { target: indicator; visible: true }
                                PropertyChanges { target: image; visible: true }
                            },
                            State {
                                name: "finished"
                                PropertyChanges { target: indicator; visible: false }
                                PropertyChanges { target: image; visible: true }
                            }
                        ]
                    }
                    BasicLabel {
                        Style.class: "shiftCloseProcess"
                        Layout.preferredHeight: 32
                        Layout.fillWidth: true
                        text: description
                    }
                }
            }
        }
    }
    footer: BottomPanel{}
}
