import QtQuick
import QtQuick.Controls
import QtQuick.Layouts

import "../components"
import "view"

import Artix.Handlers
import Artix.Models
import Artix.Core
import Artix.Gui

/**
 * @brief Форма просмотра информации о товаре
 */
BasicView {
    readonly property var document: Utils.variantToObjectPointer(params.document)
    readonly property var position: document.goodsItems[0]
    readonly property var tmc: Utils.variantToObjectPointer(params.tmc)
    readonly property string image: {
        let img = Utils.getImage(tmc.barcode)
        if (img.length > 0) {
            hasImage = true
            return img
        }
        return Style.parameter("icon.goodsPlaceholder")
    }
    property bool hasImage: false
    readonly property bool fullScreen: true

    implicitWidth: screenWidth
    implicitHeight: screenHeight

    component InfoBox: GroupBox {
        Style.class: "tmcInfoCommonData"
        interactive: false
        leftPadding: 32
        topPadding: 24
        rightPadding: 32
        bottomPadding: 24
    }

    // Полноразмерная картинка
    Popup {
        id: fullImage
        width: parent.width
        height: parent.height
        padding: 32
        modal: true
        closePolicy: Popup.CloseOnPressOutside
        background: Item {}
        Overlay.modal: Rectangle { color: Style.parameter("color.modalShading") }
        contentItem: Image {
            source: image
            fillMode: Image.PreserveAspectFit
            TapHandler { onTapped: fullImage.close() }
        }
    }

    contentItem: RowLayout {
        spacing: 0
        Rectangle {
            id: contentRect
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            Flickable {
                anchors.fill: parent
                contentHeight: implicitContentHeight + 24
                interactive: contentHeight > (contentRect.height)
                ColumnLayout {
                    anchors {
                        left: parent.left; right: parent.right; top: parent.top
                        leftMargin: 24; topMargin: 16; rightMargin: 24
                    }
                    spacing: 16
                    BasicLabel {
                        Style.class: "tmcInfoTitle"
                        Tr { code: "tmcInfoFormTitle"; defaultMessage: "Информация о товаре" }
                    }
                    Item { Layout.preferredHeight: 16 }
                    RowLayout {
                        spacing: 16
                        Image {
                            Layout.preferredWidth: 120
                            Layout.maximumHeight: 184
                            source: image
                            fillMode: Image.PreserveAspectFit
                            TapHandler {
                                enabled: hasImage && !fullImage.opened
                                onTapped: fullImage.open()
                            }
                        }
                        BasicLabel{
                            Style.class: "tmcInfoNameValue"
                            Layout.fillWidth: true
                            text: tmc.name
                        }
                        GroupBox {
                            Style.class: "tmcInfoMainData"
                            Layout.preferredWidth: 256
                            Layout.preferredHeight: 184
                            interactive: false
                            leftPadding: 32
                            topPadding: 24
                            rightPadding: 32
                            bottomPadding: 24
                            contentItem: ColumnLayout {
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormTmcPrice"; defaultMessage: "Цена" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoPriceValue"
                                    text: Utils.formatMoneyItem(position.sumb)
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormTmcRemain"; defaultMessage: "Остаток" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonValue"
                                    text: Utils.weightStr(tmc.remain)
                                }
                            }
                        }
                    }
                    RowLayout {
                        spacing: 16
                        InfoBox {
                            Layout.fillWidth: true
                            Layout.horizontalStretchFactor: 2
                            Layout.preferredHeight: 104
                            contentItem: ColumnLayout {
                                implicitWidth: Math.max(...children.map((c) => c.implicitWidth))
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormTmcCode"; defaultMessage: "Код" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonValue"
                                    Layout.maximumWidth: parent.width
                                    text: tmc.code
                                    maximumLineCount: 1
                                    elide: Text.ElideRight
                                }
                            }
                        }
                        InfoBox {
                            Layout.fillWidth: true
                            Layout.horizontalStretchFactor: 5
                            Layout.preferredHeight: 104
                            contentItem: ColumnLayout {
                                implicitWidth: Math.max(...children.map((c) => c.implicitWidth))
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormTmcBarcode"; defaultMessage: "Штрихкод" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonValue"
                                    Layout.maximumWidth: parent.width
                                    text: tmc.barcode
                                    maximumLineCount: 1
                                    elide: Text.ElideRight
                                }
                            }
                        }
                        InfoBox {
                            Layout.fillWidth: true
                            Layout.horizontalStretchFactor: 6
                            Layout.preferredHeight: 104
                            contentItem: ColumnLayout {
                                implicitWidth: Math.max(...children.map((c) => c.implicitWidth))
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormTmcArticul"; defaultMessage: "Артикул" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonValue"
                                    Layout.maximumWidth: parent.width
                                    text: tmc.articul
                                    maximumLineCount: 1
                                    elide: Text.ElideRight
                                }
                            }
                        }
                    }
                    InfoBox {
                        id: remainsBox
                        Layout.fillWidth: true
                        RemainsModelQml { id: remainsModel; tmc: params.tmc }
                        property list<real> columnsWidth: [ remainsTable.width - 120, 120 ]
                        visible: remainsModel.rowCount() > 0
                        contentItem: ColumnLayout {
                            spacing: 0
                            BasicLabel {
                                Style.class: "tmcInfoTableLabel"
                                Tr { code: "tmcInfoFormRemainsTitle"; defaultMessage: "Остатки в других магазинах" }
                            }
                            Item { Layout.preferredHeight: 8 }
                            HorizontalHeaderView {
                                id: remainsHeader
                                Layout.fillWidth: true
                                Layout.preferredHeight: 30
                                model: remainsModel
                                clip: true
                                resizableColumns: false
                                columnWidthProvider: function(column) { return remainsBox.columnsWidth[column] }
                                rowHeightProvider: function(row) { return 30 }
                                interactive: false
                                delegate: BasicLabel {
                                    Style.class: "tmcInfoColumnLabel"
                                    padding: 0
                                    leftPadding: column === 0 ? 24 : 0
                                    rightPadding: column === 1 ? 24 : 0
                                    verticalAlignment: Label.AlignVCenter
                                    horizontalAlignment: column === 0 ? Label.AlignLeft : Label.AlignRight
                                    Tr { tr: remainsModel.headerData(column, Qt.Horizontal) }
                                }

                                // FIXME: модель шлет сигнал modelReset, из-за чего виджет пересоздается.
                                // Но после удаления старый виджет продолжает отображаться на экране, причем в неправильной позиции.
                                // Из-за этого возникает задвоение заголовков. Установка `visible = false` при удалении - костыль
                                Component.onDestruction: () => { visible = false }
                            }
                            Line {
                                Layout.fillWidth: true
                                Layout.preferredHeight: strokeWidth
                                strokeWidth: 2
                                color: Style.parameter("color.grayDark")
                            }
                            TableView {
                                id: remainsTable
                                Layout.fillWidth: true
                                Layout.preferredHeight: rows * 52 + (rows - 1)
                                rowSpacing: 1
                                columnWidthProvider: function(column) { return remainsBox.columnsWidth[column] }
                                rowHeightProvider: function(row) { return 52 }
                                model: remainsModel
                                delegate: BasicLabel {
                                    Style.class: "tmcInfoTableValue"
                                    leftPadding: column === 0 ? 24 : 0
                                    rightPadding: column === 1 ? 24 : 0
                                    verticalAlignment: Label.AlignVCenter
                                    horizontalAlignment: column === 0 ? Label.AlignLeft : Label.AlignRight
                                    text: column === 0 ? display
                                                       : (tmc.tmcUnit.fractional ? Utils.weightStr(display)
                                                                                 : display)
                                    Line {
                                        anchors.top: parent.bottom
                                        width: parent.width
                                        strokeWidth: 1
                                        color: Style.parameter("color.grayBorder")
                                    }
                                }
                            }
                        }
                    }
                    InfoBox {
                        id: discountsBox
                        Layout.fillWidth: true
                        DiscountListModel { id: discountsModel; document: params.document }
                        visible: discountsModel.rowCount() > 0
                        contentItem: ColumnLayout {
                            BasicLabel {
                                Style.class: "tmcInfoTableLabel"
                                Tr { code: "tmcInfoFormDiscountsTitle"; defaultMessage: "Акции" }
                            }
                            Item { Layout.preferredHeight: 8 }
                            BasicLabel {
                                Style.class: "tmcInfoColumnLabel"
                                Layout.fillWidth: true
                                Layout.preferredHeight: 30
                                topPadding: 0
                                bottomPadding: 0
                                leftPadding: 24
                                rightPadding: 24
                                verticalAlignment: Label.AlignVCenter
                                Tr { code: "tmcInfoFormDiscountName"; defaultMessage: "Название" }
                            }
                            Line {
                                Layout.fillWidth: true
                                Layout.preferredHeight: strokeWidth
                                strokeWidth: 2
                                color: Style.parameter("color.grayDark")
                            }
                            ListView {
                                id: discountsList
                                Layout.fillWidth: true
                                Layout.preferredHeight: count * 52 + (count - 1)
                                model: discountsModel
                                spacing: 1
                                delegate: BasicLabel {
                                    Style.class: "tmcInfoTableValue"
                                    leftPadding: 24
                                    rightPadding: 24
                                    height: 52
                                    width: discountsList.width
                                    verticalAlignment: Label.AlignVCenter
                                    text: display
                                    Line {
                                        anchors.top: parent.bottom
                                        width: parent.width
                                        strokeWidth: 1
                                        color: Style.parameter("color.grayBorder")
                                    }
                                }
                            }
                        }
                    }
                    RowLayout {
                        spacing: 16
                        InfoBox {
                            Layout.fillWidth: true
                            Layout.horizontalStretchFactor: 1
                            Layout.preferredHeight: 104
                            contentItem: ColumnLayout {
                                implicitWidth: Math.max(...children.map((c) => c.implicitWidth))
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormTmcPriceNoDisc"; defaultMessage: "Цена из карточки товара" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonValue"
                                    Layout.maximumWidth: parent.width
                                    text: Utils.formatMoneyItem(tmc.price)
                                    maximumLineCount: 1
                                    elide: Text.ElideRight
                                }
                            }
                        }
                        InfoBox {
                            Layout.fillWidth: true
                            Layout.horizontalStretchFactor: 1
                            Layout.preferredHeight: 104
                            contentItem: ColumnLayout {
                                implicitWidth: Math.max(...children.map((c) => c.implicitWidth))
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormTmcMinPrice"; defaultMessage: "Минимальная цена" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonValue"
                                    Layout.maximumWidth: parent.width
                                    text: Utils.formatMoneyItem(tmc.minPrice)
                                    maximumLineCount: 1
                                    elide: Text.ElideRight
                                }
                            }
                        }
                        InfoBox {
                            Layout.fillWidth: true
                            Layout.horizontalStretchFactor: 1
                            Layout.preferredHeight: 104
                            contentItem: ColumnLayout {
                                implicitWidth: Math.max(...children.map((c) => c.implicitWidth))
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormBasePrice"; defaultMessage: "Базовая цена" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonValue"
                                    Layout.maximumWidth: parent.width
                                    text: Utils.formatMoneyItem(tmc.basePrice)
                                    maximumLineCount: 1
                                    elide: Text.ElideRight
                                }
                            }
                        }
                    }
                    RowLayout {
                        spacing: 16
                        InfoBox {
                            Layout.fillWidth: true
                            Layout.horizontalStretchFactor: 3
                            Layout.preferredHeight: 104
                            contentItem: ColumnLayout {
                                implicitWidth: Math.max(...children.map((c) => c.implicitWidth))
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormTmcDeptCode"; defaultMessage: "Код отдела" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonValue"
                                    Layout.maximumWidth: parent.width
                                    text: tmc.deptCode
                                    maximumLineCount: 1
                                    elide: Text.ElideRight
                                }
                            }
                        }
                        InfoBox {
                            Layout.fillWidth: true
                            Layout.horizontalStretchFactor: 6
                            Layout.preferredHeight: 104
                            contentItem: ColumnLayout {
                                implicitWidth: Math.max(...children.map((c) => c.implicitWidth))
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormTmcGroup"; defaultMessage: "Группа" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonValue"
                                    Layout.maximumWidth: parent.width
                                    text: tmc.tmcGroup
                                    maximumLineCount: 1
                                    elide: Text.ElideRight
                                }
                            }
                        }
                        InfoBox {
                            Layout.fillWidth: true
                            Layout.horizontalStretchFactor: 4
                            Layout.preferredHeight: 104
                            contentItem: ColumnLayout {
                                implicitWidth: Math.max(...children.map((c) => c.implicitWidth))
                                BasicLabel {
                                    Style.class: "tmcInfoCommonLabel"
                                    Tr { code: "tmcInfoFormTmcMeasureName"; defaultMessage: "Единица измерения" }
                                }
                                BasicLabel {
                                    Style.class: "tmcInfoCommonValue"
                                    Layout.maximumWidth: parent.width
                                    text: tmc.tmcUnit.name
                                    maximumLineCount: 1
                                    elide: Text.ElideRight
                                }
                            }
                        }
                    }
                }
                ScrollBar.vertical: ScrollBar {
                    id: scrollBar
                    Timer {
                        id: scrollTimer
                        interval: 2500
                        onTriggered: parent.active = false
                    }
                    onActiveChanged: scrollTimer.start()
                }
            }
        }
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 8
                BasicButton {
                    Style.class: "tmcInfoCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    Tr { code: "tmcInfoFormCancel"; defaultMessage: "Закрыть" }
                    onClicked: handler.sendAnswer(false)
                }
                BasicButton {
                    Style.class: "tmcInfoOk"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    iconLayout.alignment: Qt.AlignTop | Qt.AlignRight
                    Tr { code: "tmcInfoFormOk"; defaultMessage: "Выбрать" }
                    icon: Style.parameter("icon.actionPanel.arrow")
                    onClicked: handler.sendAnswer(true)
                }
            }
        }
    }

    footer: BottomPanel {}

    DialogHandler { id: handler }
}
