import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes

import "../../../components"
import "../"

import Artix.Core
import Artix.Gui

/**
 * @brief Панель оплаты бонусами
 */
ColumnLayout {
    id: root
    spacing: 16

    signal back()
    signal commit(var action, var args)
    property var action
    readonly property TextField inputField: inputField

    // Поле ввода
    BasicTextField {
        id: inputField
        Layout.fillWidth: true
        Layout.preferredHeight: 64
        Style.class: "documentSubtotalInput"
        padding: 12
        verticalAlignment: Text.AlignVCenter
        horizontalAlignment: Text.AlignRight
        hideCursor: true
        fixedCursorPosition: true
        validator: DoubleValidator {
            notation: DoubleValidator.StandardNotation
            bottom: 0.0
            decimals: 2
        }
    }

    Item {
        Layout.fillWidth: true
        Layout.preferredHeight: 90
        ColumnLayout {
            anchors.fill: parent
            spacing: 0
            BasicLabel {
                Style.class: "bonusTitleName"
                Tr { code: "documentSubtotalTitleBonus"; defaultMessage: "Бонусы" }
            }
            RowLayout {
                BasicLabel {
                    Style.class: "bonusOperationName"
                    Tr { code: "documentSubtotalBalanceBonus"; defaultMessage: "Сумма накоплений" }
                }
                Line {
                    Layout.alignment: Qt.AlignBottom
                    Layout.bottomMargin: 5
                    Layout.fillWidth: true
                    strokeStyle: ShapePath.DashLine
                    dashPattern: [2, 3]
                    color: Style.parameter("color.gray")
                }
                BasicLabel {
                    Layout.preferredHeight: 20
                    Style.class: "bonusOperationAmount"
                    text: Utils.formatMoneyItem(card.bonusBalance)
                }
            }
            RowLayout {
                BasicLabel {
                    Style.class: "bonusOperationName"
                    Tr { code: "documentSubtotalAvailableSpentBonus"; defaultMessage: "Доступно к списанию" }
                }
                Line {
                    Layout.alignment: Qt.AlignBottom
                    Layout.bottomMargin: 5
                    Layout.fillWidth: true
                    strokeStyle: ShapePath.DashLine
                    dashPattern: [2, 3]
                    color: Style.parameter("color.gray")
                }
                BasicLabel {
                    Layout.preferredHeight: 20
                    Style.class: "bonusOperationAmount"
                    text: Utils.formatMoneyItem(Session.document.firstActiveCard.pointsForSpend)
                }

            }
        }
    }

    Item {
        Layout.fillHeight: true
    }
    // Клавиатура и панель действий
    Item {
        Layout.fillWidth: true
        Layout.preferredHeight: 303 + (screenHeight - 768) * (0.166 * actions.rows)
        GridLayout {
            id: actions
            anchors.fill: parent
            rowSpacing: 8
            columnSpacing: 8
            columns: 2
            rows: 4

            readonly property real cellWidth: (parent.width - columnSpacing * (columns - 1)) / columns // Ширина кнопки
            readonly property real cellHeight: (parent.height - rowSpacing * (rows - 1)) / rows        // Высота кнопки

            BasicButton {
                Style.class: "actionPanelDelegate24px600"
                Layout.preferredWidth: actions.cellWidth
                Layout.preferredHeight: actions.cellHeight * Layout.rowSpan + actions.rowSpacing * (Layout.rowSpan - 1)
                Layout.row: 0
                Layout.rowSpan: 2
                Layout.column: 1

                textLayout {
                    fillWidth: true
                    fillHeight: true
                }
                iconLayout {
                    alignment: Qt.AlignRight | Qt.AlignTop
                    preferredWidth: 32
                    preferredHeight: 32
                }
                verticalAlignment: Text.AlignBottom
                Tr { code: "actionPanelButtonBack"; defaultMessage: "Назад" }
                onClicked: root.commit(root.action, {})
            }
            BasicButton {
                Style.class: "actionPanelDelegateContinue"
                Layout.preferredWidth: actions.cellWidth
                Layout.preferredHeight: actions.cellHeight * Layout.rowSpan + actions.rowSpacing * (Layout.rowSpan - 1)
                Layout.row: 2
                Layout.rowSpan: 2
                Layout.column: 1

                textLayout {
                    fillWidth: true
                    fillHeight: true
                }
                iconLayout {
                    alignment: Qt.AlignRight | Qt.AlignTop
                    preferredWidth: 32
                    preferredHeight: 32
                }
                verticalAlignment: Text.AlignBottom
                icon: Style.parameter("icon.actionPanel.arrow")
                Tr { code: "actionPanelButtonAdd"; defaultMessage: "Добавить" }
                onClicked: {
                    let args = {}
                    args["data"] = inputField.text
                    root.commit(root.action, args)
                }
            }
            InlineKeyboard {
                Style.class: "actionPanelDelegate"
                Layout.fillHeight: true
                Layout.preferredWidth: actions.cellWidth
                Layout.row: 0
                Layout.rowSpan: 4
                Layout.column: 0
                layoutType: "digits"
            }
        }
    }
}
