import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes

import "../../../components"
import Artix.Gui
import Artix.Core

/**
 * @brief Делегат товара в списке позиций
 */
PositionDelegate {
    id: gi
    property bool hasImage: image
    height: (selected ? contentItem.implicitHeight : giCommonContent.implicitHeight) + topPadding + bottomPadding
            + (object.hasConsultant ? 24 : 0)
    contentItem: ColumnLayout {
        spacing: 0
        ColumnLayout {
            visible: object.hasConsultant
            Layout.bottomMargin: 12
            RowLayout {
                Image {
                    Layout.preferredWidth: 12
                    Layout.preferredHeight: 14
                    source: Style.parameter("icon.position.consultant")
                }
                BasicLabel {
                    Layout.fillWidth: true
                    color: Style.parameter("color.info")
                    text: consultantName ? consultantName : ""
                    elide: Text.ElideRight
                    wrapMode: Text.WordWrap
                    maximumLineCount: 1
                }
            }
        }
        RowLayout {
            id: giCommonContent
            spacing: 12
            Layout.alignment: Qt.AlignTop
            Image {
                Layout.preferredWidth: 120
                Layout.preferredHeight: 120
                fillMode: Image.PreserveAspectFit
                visible: gi.hasImage
                source: gi.hasImage ? image : ""
            }
            ColumnLayout {
                spacing: 16
                RowLayout {
                    spacing: 0
                    BasicLabel {
                        Layout.fillWidth: true
                        Layout.alignment: Qt.AlignTop
                        Layout.preferredHeight: gi.hasImage ? 84 : 40
                        Style.class: "positionDelegateName"
                        text: object.name
                        elide: Text.ElideRight
                    }
                    ColumnLayout {
                        Layout.alignment: Qt.AlignTop
                        BasicLabel {
                            Layout.preferredWidth: 138
                            Layout.preferredHeight: 21
                            Style.class: "positionDelegateSum"
                            text: Utils.formatMoneyItem(object.sumb)
                            horizontalAlignment: Text.AlignRight
                        }
                        ListView {
                            Layout.preferredHeight: 24
                            Layout.preferredWidth: 138
                            layoutDirection: ListView.RightToLeft
                            orientation: ListView.Horizontal
                            spacing: 4
                            model: tags
                            delegate: Image {
                                width: 24
                                height: 24
                                fillMode: Image.PreserveAspectFit
                                source: Style.parameter(url, url)
                            }
                        }
                    }
                }
                RowLayout {
                    spacing: 0
                    BasicLabel {
                        Style.class: "positionDelegateQuant"
                        Layout.preferredHeight: 20
                        text: {
                            let quant = object.unitFlag ? Utils.weightStr(object.bquant) : object.bquant.toFixed(0)
                            return quant + " x " + Utils.formatMoneyItem(object.price)
                        }
                    }
                    BasicLabel {
                        Style.class: "positionDelegateDisc"
                        Layout.preferredHeight: 20
                        visible: !Utils.moneyIsZero(object.disc_abs + object.bonusImpactSum)
                        text: " - " + Utils.formatMoneyItem(object.disc_abs + object.bonusImpactSum)
                    }
                    Item { Layout.preferredWidth: 12 }
                    BasicLabel {
                        Style.class: "positionDelegateDiscName"
                        elide: Text.ElideRight
                        Layout.fillWidth: true
                        Layout.preferredHeight: 20
                        padding: 8
                        topPadding: 1
                        bottomPadding: 3
                        visible: discountName !== undefined
                        text: discountName ? discountName : ""
                    }
                }
            }
        }
        ColumnLayout {
            id: giSelectedContent
            opacity: selected ? 1 : 0
            spacing: 0
            Item { Layout.preferredHeight: 16; visible: campaingName !== undefined }
            BasicLabel {
                Style.class: "positionDelegateCampaignTitle"
                Layout.fillWidth: true
                Layout.preferredHeight: 16
                visible: campaingName !== undefined
                Tr { code: "positionDiscount"; defaultMessage: "Акция" }
            }
            Item { Layout.preferredHeight: 4; visible: campaingName !== undefined }
            BasicLabel {
                Style.class: "positionDelegateCampaignName"
                Layout.fillWidth: true
                Layout.preferredHeight: 20
                elide: Text.ElideRight
                visible: campaingName !== undefined
                text: campaingName ? campaingName : ""
            }
            Item { Layout.preferredHeight: 24 }
            BasicButton {
                visible: object.hasConsultant
                Layout.fillWidth: true
                Layout.preferredHeight: 40
                Style.class: "consultantDelete"
                textLayout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                spacing: 16
                Tr { code: "positionConsultantDelete"; defaultMessage: "Удалить консультанта" }
                onClicked: consultantDelete(position)
            }
            Item {
                visible: object.hasConsultant
                Layout.preferredHeight: 24
            }
            RowLayout {
                Layout.fillWidth: true
                Layout.preferredHeight: 56
                spacing: 16
                BasicButton {
                    Layout.preferredWidth: 72
                    Layout.preferredHeight: 56
                    Style.class: "positionQuant"
                    icon: Style.parameter("icon.minus")
                    enabled: Utils.weightGreater(object.bquant, 1.0) && object.unitFlag !== true
                    onClicked: changeQuantity(position, -1.0)
                }
                BasicButton {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 56
                    Style.class: "positionStorno"
                    textLayout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
                    Tr { code: "positionStorno"; defaultMessage: "Удалить" }
                    onClicked: storno(position)
                }
                BasicButton {
                    Layout.preferredWidth: 72
                    Layout.preferredHeight: 56
                    Style.class: "positionQuant"
                    icon: Style.parameter("icon.plus")
                    enabled: object.unitFlag !== true
                    onClicked: changeQuantity(position, 1.0)
                }
            }
            Behavior on opacity { OpacityAnimator { duration: 500 } }
        }
    }
}
