import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes

import "../../../components"
import Artix.Gui
import Artix.Core
import Artix.Models

/**
 * @brief Делегат модификаторов в списке позиций
 */
PositionDelegate {
    height: (selected ? contentItem.implicitHeight : modifiersCommonContent.implicitHeight) + topPadding + bottomPadding
    contentItem: ColumnLayout {
        spacing: 40
        RowLayout {
            id: modifiersCommonContent
            spacing: 12
            Layout.preferredWidth: parent.width

            Repeater {
                id: modifiersRepeater
                model: ModifiersModel { modifiers: object }

                delegate: RowLayout {
                    Layout.preferredWidth: parent.width / modifiersRepeater.count
                    Layout.maximumWidth: Layout.preferredWidth
                    Layout.preferredHeight: implicitHeight > parent.height ? implicitHeight : parent.height
                    spacing: modifiersCommonContent.spacing
                    property bool isLast: index === modifiersRepeater.count - 1
                    property bool showIcon: true
                    onWidthChanged: showIcon = modifiersRepeater.count < 3 && (width >= (implicitWidth + (showIcon ? 0 : 24 + spacing)))

                    ColumnLayout {
                        Layout.preferredHeight: implicitHeight > parent.height ? implicitHeight : parent.height
                        spacing: 16
                        BasicLabel {
                            Layout.alignment: Qt.AlignTop
                            Style.class: "modifierText"
                            Tr { tr: name }
                        }
                        BasicLabel {
                            visible: type !== ModifierType.PackingMode
                            Style.class: "modifierValue"
                            text: value
                        }
                    }

                    Image {
                        visible: showIcon
                        Layout.alignment: Qt.AlignTop | Qt.AlignRight
                        Layout.preferredWidth: 24
                        Layout.preferredHeight: 24
                        source: Style.parameter(icon)
                    }

                    Line {
                        visible: !isLast
                        direction: LineDirection.Vertical
                        strokeWidth: 1
                        strokeStyle: ShapePath.DashLine
                        dashPattern: [2, 2]
                        color: Style.parameter("color.gray")

                        Layout.fillHeight: true
                        Layout.alignment: Qt.AlignRight
                    }
                }
            }
        }
        BasicButton {
            Layout.fillWidth: true
            Layout.preferredHeight: 56
            Layout.leftMargin: 88
            Layout.rightMargin: 88
            opacity: selected ? 1 : 0
            Style.class: "positionStorno"
            textLayout.alignment: Qt.AlignVCenter | Qt.AlignHCenter
            Tr { code: "positionStorno"; defaultMessage: "Удалить" }
            onClicked: handler.emitAction("COMMAND_ESC", {})
            Behavior on opacity { OpacityAnimator { duration: 500 } }
        }
    }
}
